use based::auth::{Sessions, User};
use based::request::StringResponse;
use based::ui::prelude::*;
use based::{auth::MaybeUser, request::RequestContext};
use maud::html;
use rocket::http::CookieJar;
use rocket::{form::Form, get, http::Cookie, post, response::Redirect, FromForm};

use crate::library::history::VideoHistory;
use crate::pages::components::video_element_wide;

use super::components::render_page;

#[get("/login")]
pub async fn login(ctx: RequestContext, user: MaybeUser) -> StringResponse {
    let content = html!(
        h2 { "Login" };
        form action="/login" method="POST" {
            input type="text" name="username" placeholder="Username" required;
            input type="password" name="password" placeholder="Password" required;
            input type="submit" value="Login";
        }
    );

    render_page(ctx, content, "Login", user.into()).await
}

#[derive(FromForm)]
pub struct LoginForm {
    username: String,
    password: String,
}

#[post("/login", data = "<login_form>")]
pub async fn login_post(login_form: Form<LoginForm>, cookies: &CookieJar<'_>) -> Option<Redirect> {
    let login_data = login_form.into_inner();

    let (session, _) = User::login(&login_data.username, &login_data.password).await?;

    let session_cookie = Cookie::build(("session", session.token.to_string()))
        .path("/") // Set the cookie path to the root so it’s available for the whole app
        .http_only(true) // Make the cookie HTTP only for security
        .max_age(rocket::time::Duration::days(7)) // Set the cookie expiration (7 days in this case)
        .build();

    cookies.add(session_cookie);

    Some(Redirect::to("/"))
}

#[get("/history")]
pub async fn history_page(ctx: RequestContext, user: User) -> StringResponse {
    let content = html! {
        h1 class="text-center text-4xl font-extrabold leading-tight mt-4 mb-2" { "History" };
            (Padding(html! {
                @for mut vid in user.history_of(10).await {
                    ( video_element_wide(&mut vid).await );
                };
            }).all(ScreenValue::_6))
    };

    render_page(ctx, content, "History", Some(user)).await
}
